
package org.ow2.petals.registry.database;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;

import org.ow2.petals.util.LoggingUtil;


public class RegistryInitialContextFactory implements javax.naming.spi.InitialContextFactory {

    private final static String POOLSIZE = "java.naming.poolsize";

    protected final static String DEFAULT_USERNAME = "sa";

    protected final static String DEFAULT_PASSWORD = "";

    protected final static int DEFAULT_POOLSIZE = 5;

    protected final static int DEFAULT_BATCHSIZE = 10;

    private final static String JAVA_NAMING_FACTORY_LOGGER = "java.naming.factory.logger";

    public Context getInitialContext(final Hashtable<?, ?> environment)
    throws NamingException {

        final LoggingUtil log = (LoggingUtil) environment
        .get(JAVA_NAMING_FACTORY_LOGGER);
        log.info("Initializing a PostgreSQL initial context");

        final String urlStr = (String) environment.get(Context.PROVIDER_URL);
        // Check that the URL is well-formed
        try {
            new URI(urlStr);
        } catch (final URISyntaxException e) {
            final NamingException nex = new NamingException(e.getMessage());
            nex.setRootCause(e);
            throw nex;
        }
        log.info("  - PostgreSQL connection URL: " + urlStr);

        String username = (String) environment.get(Context.SECURITY_PRINCIPAL);
        if (username == null || username.length() == 0) {
            username = DEFAULT_USERNAME;
        }
        log.info("  - PostgreSQL connection username: " + username);

        String password = (String) environment
        .get(Context.SECURITY_CREDENTIALS);
        if (password == null || password.length() == 0) {
            password = DEFAULT_PASSWORD;
        }

        final String connectionPoolSizeStr = (String) environment.get(POOLSIZE);
        int connectionPoolSize = DEFAULT_POOLSIZE;
        if (connectionPoolSizeStr != null
                && connectionPoolSizeStr.length() != 0) {
            connectionPoolSize = Integer.parseInt(connectionPoolSizeStr);
        }
        log.info("  - PostgreSQL connection pool size: " + connectionPoolSize);

        final String batchsizeStr = (String) environment.get(Context.BATCHSIZE);
        int batchSize = DEFAULT_BATCHSIZE;
        if (batchsizeStr != null && batchsizeStr.length() != 0) {
            batchSize = Integer.parseInt(batchsizeStr);
        }
        log.info("  - PostgreSQL connection batchsize: " + batchSize);

        log.info("PostgreSQL initial context initialized.");

        return new RegistryContextImpl(urlStr, username, password,
                connectionPoolSize, batchSize,
                (LoggingUtil) environment
                .get(JAVA_NAMING_FACTORY_LOGGER));
    }

}
